#ifndef OFSTREAM_HPP_
#define OFSTREAM_HPP_
#define ofstream_hpp_version "$Revision: 1.1 $"

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <string>




/**
 * Class to write to an ASCI-File.
 */
class OFStream{
	/** Pointer to the file to write to*/
	FILE* 	out;

	/** Indicates whether 'out' shall be closed at destruction. */
	bool close_at_destruct;

	/** Stores the pattern that is sort of background for each writeString operation.*/
	char* pattern;

	/** Length of the pattern or -1 if no pattern is specified.*/
	int pattern_length;

public:
	/** Creates an invalid OFStream.*/
	inline OFStream();

	/** Initialisation with the name of the file to write.
	 * @param file_name: name of the file
	 * @param overwrite: if true, an existing file will be overwritten. Default is true.
	 * @param close_at_destruction: if true, the file is closed at destruction. Default is true.*/
	inline OFStream(const char* file_name,bool overwrite = true, bool close_at_destruction = true);

	/** Initialisation with the file object to write. The file is not closed at destruction.
	 * It is also possible to specify 'stdout' or 'stderr' here.
	 * @param file: the file object to be written to*/
	inline OFStream(FILE* file);

	/** Close the underlying file and destruct this object.*/
	inline ~OFStream();

	/** Close the underlying file.*/
	inline void close();

	/** Checks if the file is open for writing.
	 * @return true, if the file cannot be written. */
	inline bool isBad()        const;

	/** Checks if the file is empty.
	* @return true, if the internal file pointer for reading is at EOF.*/
	inline bool isEmpty()      const;

	/** Checks if the file is open for writing.
	* @return true, if the file can be written. */
	inline bool isValid()      const;

	/** Print a new line.*/
	inline void writeNewLine();

	/** Print a null terminated string.
	 * @param format: the format string to be written
	 * @param ...: a variable number of arguments to the format string
	 * @return true iff the string could be written*/
	inline bool writeString(const char* format, ...);

	inline bool writeString(const std::string& std);


	inline bool writeCenteredToBG(const char* str);


	/** Set a background pattern for each writeString operation.
	 * If the pattern is set to "********" an invokation 'writeString("N:%i",15)'
	 * would print the following string: "N:15****". The pattern will be ignored in case
	 * the printed string (specified in writeString) becomes longer than the background pattern,.
	 * @param p: a background pattern.*/
	inline void setBackgroundPattern(const char* p);

	/** Deactivate the background pattern for subsequent writeString operations.
	 * The pattern can be reactivated by a call to the function reactivateBackgroundPattern.*/
	inline void deactivateBackgroundPattern();

	/** Reactivate a previously deactivated valid background pattern.*/
	inline void reactivateBackgroundPattern();


private:

	/** Prohibited: Copying of OFStream.*/
	OFStream(const OFStream& cp);

	/** Prohibited: Assignment of OFStream.*/
	OFStream& operator=(const OFStream& cp);

};




//Implementation
inline 		OFStream::OFStream()					:out(NULL),close_at_destruct(false),pattern(NULL),pattern_length(-1){}

inline 		OFStream::OFStream(FILE* file)			:out(file),close_at_destruct(false),pattern(NULL),pattern_length(-1){}

inline 		OFStream::~OFStream()					{if(close_at_destruct && out != NULL) fclose(out);}

inline void OFStream::close()						{if(out != NULL) fclose(out); out = NULL;}

inline bool OFStream::isBad() 	const				{return out == NULL;}

inline bool OFStream::isEmpty()	const				{return fgetc(out) == EOF;}

inline bool OFStream::isValid()	const				{return out != NULL;}

inline void OFStream::writeNewLine()				{fprintf(out,"\n"); fflush(out);}

inline void OFStream::deactivateBackgroundPattern()	{pattern_length = -1;}

inline void OFStream::reactivateBackgroundPattern()	{pattern_length = (pattern != NULL ? (int)strlen(pattern):-1);}

inline void OFStream::setBackgroundPattern(const char* p)	{pattern = (char*)p; pattern_length = strlen(pattern);}

inline 		OFStream::OFStream(const char* file_name,bool overwrite, bool close_at_destruction)
	: close_at_destruct(close_at_destruction),pattern(NULL),pattern_length(-1)
	{ out = fopen (file_name, (overwrite?"w+":"a+"));}

/*
inline bool OFStream::writeString(std::string& str){
	int written = fprintf(out,"%s",str.c_str() );

	if(pattern_length > 0 && written >= 0 && written < pattern_length){
		fprintf(out,"%s",(pattern+written));}

	fflush(out);
	return written >= 0;
}*/

inline bool OFStream::writeString(const char* format, ...){
	va_list args;
	va_start(args, format);
	int written = vfprintf(out, format, args);
	va_end(args);

	if(pattern_length > 0 && written >= 0 && written != pattern_length){
        fprintf(out,"%s",pattern + (written<pattern_length ? written:pattern_length-1));}

	fflush(out);
	return written >= 0;
}

inline bool OFStream::writeCenteredToBG(const char* str){
    if(pattern_length < 0) return false;

    int len = strlen(str);
    if(pattern_length <= len) return writeString("%s",str);

    int s = (pattern_length - len) >> 1;
    for(int i = 0; i < s; ++i) fprintf(out,"%c",(pattern[i]));
    s = s + fprintf(out,"%s",str);
    if(s < pattern_length) fprintf(out,"%s",pattern + s);

    return true;
}



#endif /*OFSTREAM_HPP_*/
